/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ajax
 *
 * @revision		$Revision: 1.1.2.1 $
 */

jQuery.plugin( 'ep.ajax', 'ep.core', function( $, undefined ){

	ep.extend({
		ajax: function( settings ){

			var jqXHR;

			if( !settings.url ){
				settings.url = ep.config.baseUrl;
			}

			if( settings.dataType === 'json' ){
				var _beforeSend = settings.beforeSend;

				// add special header for epages
				$.extend(settings,{
					beforeSend: function( xhr, s ){
						xhr.setRequestHeader("Accept", "application/json");
						if( _beforeSend ){
							_beforeSend( xhr, s );
						}
					}
				});

				// save deprecated properties as locals
				var success	= settings.success,
					error	= settings.error,
					complete= settings.complete;

				// delete deprecated properties
				settings.success = settings.error = settings.complete = undefined;

				// create AJAX object
				jqXHR = $.ajax(settings);

				// init
				var context		= settings.context || jqXHR,
					deferred	= $.Deferred(),
					complete	= $.Deferred(),
				// save original deferrd methods
					originalDone = jqXHR.done,
					originalFail = jqXHR.fail;

				// set bridge for .done() calls
				originalDone(function( data, status, jqXHR ){
					if( data && data.Errors ){
						var errorThrown = data.Errors;
						jqXHR.status = 400;
						jqXHR.statusText = status = "error";
						deferred.rejectWith( context, [jqXHR, status, errorThrown] );
					}
					else{
						deferred.resolveWith( context, [data, status, jqXHR] );
					}
				});

				// set bridge for .fail() calls
				originalFail(function( jqXHR, status, errorThrown ){
					deferred.rejectWith( context, [jqXHR, status, errorThrown] );
				});

				// set bridge for .always() calls
				deferred.always(function(){
					complete.resolveWith( context, [jqXHR] );
				});

				// map .done() to own deferrd
				jqXHR.done = function(){
					deferred.done.apply( deferred, arguments );
					return jqXHR;
				};

				// map .fail() to own deferrd
				jqXHR.fail = function(){
					deferred.fail.apply( deferred, arguments );
					return jqXHR;
				};

				// map .always() to own deferrd
				jqXHR.always = function(){
					complete.done.apply( complete, arguments );
					return jqXHR;
				};

				// restore deprecated properties from locals
				if( success ){
					jqXHR.done(success);
				}
				if( error ){
					jqXHR.fail(error);
				}
				if( complete ){
					jqXHR.always(complete);
				}
			}
			else{
				jqXHR = $.ajax(settings);
			}

			return jqXHR;
		}
	});



});
