/*
 * @copyright		© Copyright 2006-2012, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages.presentation.inc.mainMenu
 *
 * @revision		$Revision: 1.1.2.1 $
 */

jQuery.plugin('de_epages.presentation.inc.mainMenu', {
	plugin : [
		'jQuery.tmpl',
		'de_epages.core'
	],
	tmpl : [
		'de_epages.presentation.inc.mainmenu'
	]
}, function ($) {

	// Used for "delayed" *:hover* later on.
	var liWithChild;

	$.extend($.scope("de_epages.presentation.inc"), {
		mainMenu : function (selector, updates) {
			var $node = $(selector),
					menu = $node.data('menu');

			// Render template (which also handles updating menu items),
			// append to *node* and assign *onClick* handler, if so desired.
			$.tmpl('de_epages.presentation.inc.mainmenu', menu, {
				'updates' : updates,
				// Setup class attribute of menu item's *li* node.
				'class' : function () {
					var self = this,
							data = self.data,
							classAttr = '',
							status;

					if (data.children && data.children.length) {classAttr += 'hasChildren';}
					if (self.followedBySeparator) {classAttr += (classAttr.length ? ' ' : '') + 'Separator';}
					switch (data.status) {
						case 'DISABLED' : classAttr += (classAttr ? ' ' : '') + 'Disabled'; break;
						case 'FEATURE_DISABLED' : classAttr += (classAttr ? ' ' : '') + 'FeatureDisabled'; break;
						case 'HIDDEN' : classAttr += (classAttr ? ' ' : '') + 'Hidden'; break;
						case 'SELECTED' : classAttr += (classAttr ? ' ' : '') + 'Selected'; break;
						default : break;
					}
					if (!classAttr) {return '';}
					else {return 'class="' + classAttr + '"';}
				},
				// Update menu item.
				'setup' : function () {
					var self = this,
							data = self.data;

					if (update=self.updates[data.menuId]) {
						$.extend(data, update);
					}
					return "";
				}
			})
			.appendTo($node)
			.find('.ep-menu-click').each(function (i) {
				var $this = $(this);
				$this.on('click', $this.tmplItem('data')['onClick'])
			});

			// Simulate a delayed *:hover* status with *jQuery.delay* and the *ui-hover* class.
			liWithChild = $node.find('li.hasChildren');

			liWithChild
				.on('mouseleave', function (event) {
					// Remove *ui-hover* class after *delay*.
					$(this)
						.delay(250)
						.queue(function (next) {
							$(this).removeClass('ui-hover');
							next();
						});
				})
				.on('mouseenter', function (event) {
					// Remove *ui-hover* from all *li.hasChildren.ui-hover* elements
					// except maybe *this* and elements containing *this*.
					liWithChild
						.filter('.ui-hover')
						.not(this)
						.not(function(){
							return $.contains(this, event.target)
						})
						.clearQueue()
						.removeClass('ui-hover');
					// Add *ui-hover* class to *this*.
					$(this)
						.clearQueue()
						.addClass('ui-hover');
				});
		}
	});
});