/*
 * @copyright		© Copyright 2006-2011, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages.remotesearch.ui.suggest
 *
 * @revision		$Revision: 1.1.2.1.2.1 $
 */

jQuery.plugin( "de_epages.remotesearch.ui.suggest", {
	plugin:	["jQuery.json", "jQuery.tmpl", "jQuery.ui.widget", "ep.core", "jQuery.dict", "ep.fn.contextOrientation", "ep.ui.validate", "de_epages.core"],
	tmpl:	["de_epages.remotesearch.ui.suggest"],
	dict:	["de_epages.remotesearch.dictionary"]
}, function($){

/*
 * @dictionary			de_epages.remotesearch.dictionary
 *
 * @translation			{Products}
 *						{Categories}
 *						{Manufacturers}
 *						{SearchFor}
 */

 	var dict		= $.dict("de_epages.remotesearch.dictionary"),
 		tmplDict	= {
			products		: dict.translate("Products"),
			categories		: dict.translate("Categories"),
			manufacturers	: dict.translate("Manufacturers"),
			searchfor		: dict.translate("SearchFor")
		};

	$.widget( "de_epages.remotesearch.ui.Suggest", ep.ui.Validate, {

		options: {
		//	searchFilter	: "selector input",
		//	suggestUrl		: "?",
			suggestChars	: 2,
			suggestMax		: 10,
			showImages		: true
		},

		_create: function(){
			this.Inherited(arguments);

			var self = this,
				o = self.options;

			// hidden input for filter options
			self.filter = o.filter ? $(o.searchFilter) : $('<input type="hidden"/>').insertAfter(self.elem);

			self.elem
				.attr("autocomplete","off")
				.on("keydown", self.Proxy("_keydown"))
				.on("keyup", self.Proxy("_keyup"))
				.on("focus", self.Proxy("_show"))
				.on("blur", self.Proxy("_hide"));

			// list container
			self.list = ep('<div class="de_epages-remotesearchUiSuggest-box"/>')
				.on('mouseenter','li',function(){
					$(this).addClass('ui-hover');
				})
				.on('mouseleave','li',function(){
					$(this).removeClass('ui-hover');
				})
				.on('mousedown','li:not(.Separator)',function( event ){
					self._select( $(this).data('index') );
					self._action( event );
					// move action in other tread
					setTimeout(function(){
						self.elem.trigger('focus');
					},1);

				});
			// caching for fast suggests
			self.cache = {};
			self.dataModels = {};
			// default selected index of suggest items
			self.selectedIndex = -1;
			// setup an object like jqXHR to provide an abort method
			self.xhr = {abort: $.noop};
			// basic shr settings
			self.xhrSettings = {
				type	: "get",
				dataType: "jsonp",
				data	: {
//					datasource	: ep.config.storeName,
//					shopGUID	: ep.config.siteGuid,
//					lang		: ep.config.language

				}
			};
			if( o.suggestUrl ){
				self.xhrSettings.url = o.suggestUrl;
			}
		},

		_keydown: function( event ){
			var keyCode = event.keyCode || event.which;
			if( keyCode == 38 || keyCode == 40 ){
				event.preventDefault();
			}
			else if( keyCode == 13 ){
				this._action( event );
			}
		},

		_keyup: function( event ){
			var self	= this,
				keyCode = event.keyCode || event.which,
				query	= self.elem.val(),
				o		= self.options;

			if( keyCode == 9 || keyCode == 13 || keyCode == 37 || keyCode == 39 ){
				return;
			}
			else if( keyCode == 38 ){
				self._select( 'prev' );
			}
			else if( keyCode == 40 ){
				self._select( 'next' );
			}
			else if( query.length < self.options.suggestChars ){
				self._clear();
			}
			else if( self.cache[ query ] ){
				self._build();
			}
			else{
				if (self.lastAjaxFail) {
					if ((new Date().getTime() - self.lastAjaxFail) < 60000) {
						// in case a request failed less than 1 minute ago
						// we will not bother the server this time.
						return;
					}
					else {
						// let us try talking to the server again now.
						self.lastAjaxFail = null;
					}
				}
				$.ajax(
					$.extend(true,{
						data:{
							q: query
						}
					}, self.xhrSettings)
				)
				.done(function( response ){
					$.extend( self.cache[ query ] || (self.cache[ query ] = {}), response );
					self._build();
				})
				.fail(function() {
					self.lastAjaxFail = new Date().getTime();
				});
			}
		},

		_hide: function(){
			this.list.hide();
		},

		_show: function(){
			if( this.list.find('li').length ){
				this.list.show();
			}
		},

		_clear: function(){
			this.list.empty();
			this._hide();
		},

		_dataModel: function( query ){
			var self = this,
				dataModel = self.dataModels[ query ],
				data;

			if (!dataModel) {
				data = self.cache[ query ];

				dataModel = {
					query	: query,
					list	: [],
					dict	: tmplDict
				};

				if (data) {
					$.each(["products", "manufacturers", "categories"], function (i, type) {
						data[type] = $.each(data[type] || [], function () {
							this.type = type;
						});

						dataModel.list = dataModel.list.concat(data[type]);
					});

					// Cache model
					self.dataModels[ query ] = dataModel;
				}
			}

			// Use current options
			dataModel.o = self.options;

			return dataModel;
		},

		_build: function(){
			var self = this,
				query = self.elem.val(),
				dataModel = self.dataModel = self._dataModel(query),
				rHighlight = new RegExp( "("+ep.escapeExpStr(query)+")", "ig");

			self._clear();

			self.list
				.appendTo("body")
				.contextOrientation(self.elem,"bottom")
				.append(
					$.tmpl(
						"de_epages.remotesearch.ui.suggest",
						[dataModel],
						{
							highlight: function( text ){
								return text.replace(rHighlight,"<strong>$1</strong>");
							},
							separatorLast: '',
							separator: function( type ){
								return this.separatorLast == type ? false : !!(this.separatorLast = type);
							}
						}
					)
				);

			self._select(-1);
			self._show();
		},

		_select: function( i ){
			var self = this,
				set = self.selectedIndex,
				items = self.list.find('li:not(.Separator)');

			// reset filter data
			self.filterData = {};

			if( i == 'next' ){
				set++;
			}
			else if( i == 'prev' ){
				set--;
			}
			else{
				set = i;
			}

			if( set >= items.length ){
				set = -1
			}
			else if( set < -1 ){
				set = items.length - 1;
			}

			items.removeClass('ui-select ui-active');
			if( set != -1 ){
				items.eq(set)
					.addClass('ui-select ui-active');

				self.filterData = (self.dataModel.list[ set ] || {}).filter || {};
			}

			if( $.type(self.filterData) === "object" ){
				self.filter
					.attr("name", self.filterData.fieldName || "" )
					.val(self.filterData.attributeValue || "" );
			}

			self.selectedIndex = set;
		},

		_action: function( event ){
			var self = this;

			event.preventDefault();

			// Special: go to page by object id
			if( $.type(self.filterData) === "number" ){
				location.href = ep.config.baseUrl + "?ObjectID=" + self.filterData;
			}
			// Default: submit form
			else{
				self.elem
					.closest('form')
					.trigger('submit');
			}
		}

	});

});