use strict;
use Test::More;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Error qw (ExistsError GetError ResetError);
use DE_EPAGES::Core::API::Log qw (LogError);
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Database::API::Connection qw (RunOnStore GetTestSite);
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias);

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $Help;
    my $NoOutput;
    my $test_ok;

    if (!$StoreName){
        $test_ok = 1;
    }

    GetOptions(
        'help'          => \$Help,
        'nooutput'      => \$NoOutput,
        'passwd=s'      => \$Password,
        'storename=s'   => \$StoreName
    );

    my $time1 = time;

    RunOnStore(
        Store       => GetTestSite(),
        DBPassword  => $Password,
        Sub         => sub  {
            # make sure that the TrustedShops cartridge is installed
            unless (ExistsCartridgeByPackage('DE_EPAGES::TrustedShops')) {
                print "Store: $StoreName : Cartridge TrustedShops is not installed.\n" unless $NoOutput;
                return;
            }
            my $aCertificates = LoadClassByAlias('TrustedShopsCertificate')->objects;
            foreach my $Certificate (@$aCertificates) {
                next if ($Certificate->getSite()->get('ClosedByProvider'));
                my $ret;
                eval {
                    $ret = updateAll(); #mockup of $Certificate->updateAll
                };
                if (ExistsError()) {
                    my $Error = GetError();
                    LogError($Error);
                    print STDERR 'update of certificate ' . $Certificate->id . ' (' . $Certificate->pathString .
                    ') failed: ' . $Error->message . "\n";
                    ResetError();
                }
                elsif (!$ret->{'code'}) { # no error message if login has been reset for a excellence certificate
                    if (!$Certificate->get('IsExcellence') || defined($Certificate->get('WebserviceUsername'))){
                        print STDERR 'update of certificate ' . $Certificate->id . ' (' . $Certificate->pathString .
                        ') failed for ' . $ret->{'message'} . ' in Store: ' . $StoreName . "\n";
                        $test_ok = 1;
                    }
                }
            }
        }
    );
    printf ("need %.3f seconds\n", time() - $time1) unless ($NoOutput);
    cmp_ok($test_ok,'==',1,'Error message OK');
    done_testing();
}

sub updateAll {
    return ({'code' => undef , 'message' => 'invalid certificate'});
}

RunScript(
    'Sub' => \&Main,
);
